import os
import sys


img_info = []

LOADER = """
import lvgl as lv
import os
import gc
import micropython

if '/' in __file__:
    BASE_PATH = __file__.rsplit('/', 1)[0] + '/'
else:
    BASE_PATH = ''


class ImageLoaader(object):

    @micropython.native
    def __getattr__(self, item):    
        if item in self.__dict__:
            return self.__dict__[item]

        mapping = {{{mapping}}}

        if item not in mapping:
            raise AttributeError(item)

        path = self._path + item + '.bin'
        header = {{'always_zero': 0}}

        header.update(mapping[item])
        file = open(path, 'rb')
        try:
            return lv.img_dsc_t({{  # NOQA
                'header': header,
                'data_size': os.stat(path)[6],
                'data': file.read()
            }}) 
        finally:
            file.close()
            gc.collect()
"""


def convert_image(src, dst, name):
    with open(src, 'rb') as file:
        data = file.read()

    if b'const lv_img_dsc_t ' not in data:
        return False

    if b'_map[] = {' not in data:
        return False

    img_data = data.split(b'_map[] = {')[-1]
    img_data = img_data.split(b'}', 1)[0]

    img_data = img_data.replace(b'/*alpha channel*/', b'')

    img_data = b'bytearray([' + img_data + b'])'

    img_data = eval(img_data)

    with open(dst, 'wb') as file:
        file.write(img_data)

    img_format = data.split(b'.header.cf = ')[-1].split(',')[0].decode('utf_8')
    img_width = data.split(b'.header.w = ')[-1].split(',')[0].decode('utf_8')
    img_height = data.split(b'.header.h = ')[-1].split(',')[0].decode('utf_8')
    img_format = img_format.replace('LV_IMG_', 'lv.img.').replace('_', '.')

    img_info.append(
        "'{name}: {{'w': {width},  'h': {height}, 'cf': {img_format}}}".format(
            name=name.replace(' ', '_'),
            width=img_width,
            height=img_height,
            img_format=img_format
        )
    )

    return True


def main(src_path=None, dst_path=None):
    if src_path is None:
        src_path = os.getcwd()

    if dst_path is None:
        dst_path = os.getcwd()
    elif not os.path.isdir(dst_path):
        raise RuntimeError('destination path must be a directory')

    if os.path.isdir(src_path):
        for f in os.listdir(src_path):
            name, ext = os.path.splitext(f)
            if ext != '.c':
                continue

            src = os.path.join(src_path, f)
            dst = os.path.join(dst_path, name + '.bin')
            convert_image(src, dst, name)

    else:
        f = os.path.split(src_path)[1]
        name, ext = os.path.splitext(f)

        if ext != '.c':
            raise RuntimeError('invalid file')

        dst = os.path.join(dst_path, name + '.bin')
        src = src_path
        if not convert_image(src, dst, name):
            raise RuntimeError('invalid file')

    dst = os.path.join(dst_path, 'image_loader.py')

    with open(dst, 'w') as file:
        file.write(LOADER.format(mapping=', '.join(img_info)))


if __name__ == '__main__':
    if len(sys.argv) > 1:
        main(*sys.argv[1:])
    else:
        main()
